#include "StdAfx.h"
#include "TelnetClient.h"
#include "TelnetServer.h"

using namespace System::Net;

cTelnetServer::cTelnetServer(void)
{
	m_objTelnetServer = this;

	m_sLoginTelnet = gcnew String("admin");
	m_sPwdTelnet = gcnew String("admin");

	m_objClientList = gcnew ArrayList;
	m_objSocket = nullptr;
}

void cTelnetServer::OnServerAccept(IAsyncResult^ ar)
{
	cTelnetServer^ objTelnetServer = dynamic_cast<cTelnetServer^>(ar->AsyncState);
	
	Monitor::Enter(m_objLock);
	
	if (objTelnetServer->m_objSocket == nullptr)
	{
		Monitor::Exit(m_objLock);
		return;
	}
                      
	cTelnetClient^ objClientSocket = gcnew cTelnetClient();

	objClientSocket->m_objSocket = objTelnetServer->m_objSocket->EndAccept(ar);
	objClientSocket->m_nClientListIndex = objTelnetServer->m_objClientList->Add(objClientSocket);
	objClientSocket->OnConnected();

	objTelnetServer->m_objSocket->BeginAccept(gcnew AsyncCallback(OnServerAccept), objTelnetServer);
	
	Monitor::Exit(m_objLock);
}

bool cTelnetServer::StartServer()
{
	StopServer();

	m_objSocket = gcnew Socket(AddressFamily::InterNetwork, SocketType::Stream, ProtocolType::Tcp);

	try
	{
		IPEndPoint^ localEndPoint = gcnew IPEndPoint(IPAddress::Any, 4332);

		m_objSocket->Bind(localEndPoint);
		m_objSocket->Listen(100);

		m_objSocket->BeginAccept(gcnew AsyncCallback(OnServerAccept), this);
	}
	catch (SocketException^ e)
	{
		return false;
	}

	return true;
}

bool cTelnetServer::IsAuth(String^ sLogin, String^ sPassword)
{
    return ((m_sLoginTelnet == sLogin) && (m_sPwdTelnet == sPassword));
}

void cTelnetServer::StopServer()
{
    Monitor::Enter(m_objLock);
    
	if (m_objSocket != nullptr)
	{
		m_objSocket->Close();
		m_objSocket = nullptr;
	}

	for (int nCount = 0; nCount < m_objClientList->Count; nCount++)
	{
		cTelnetClient^ objTelnetClient = (cTelnetClient^) m_objClientList[nCount];
		objTelnetClient->CloseSocket();
	}

    m_objClientList->Clear();

	Monitor::Exit(m_objLock);
}

void cTelnetServer::OnClientDisconnect(cTelnetClient^ objTelnetClient)
{
    m_objClientList->RemoveAt(objTelnetClient->m_nClientListIndex);
}

void cTelnetServer::SendCommandToAll(String^ sCommand, ArrayList^ objParamList)
{
    cTelnetClient^ objTelnetClient;
    
    for(int nCount = 0; nCount < m_objClientList->Count; nCount++)
    {
        objTelnetClient = (cTelnetClient^) m_objClientList[nCount];
        objTelnetClient->SendCommand(sCommand, objParamList);
    }
}